///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hbool;

import 'HVal.dart';

/// HBool defines singletons for true/false tag values.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HBool extends HVal {
  /// Factory Constructor from boolean value
  factory HBool(bool val) => val ? TRUE : FALSE;

  /// Singleton value for true
  static final HBool TRUE = new HBool._(true);

  /// Singleton value for false
  static final HBool FALSE = new HBool._(false);

  HBool._(this.val);

  /// Boolean value
  final bool val;

  /// Hash code is same as java.lang.Boolean
  @override
  int get hashCode => val ? 1231 : 1237;

  /// Encode as "true" or "false"
  @override
  String toString() => val ? "true" : "false";

  /// Raise UnsupportedOperationException
  @override
  String toJson() {
    throw new UnsupportedError("HBool.toJson()");
  }

  /// Encode as "T" or "F"
  @override
  String toZinc() => val ? "T" : "F";
}
