///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

part of haystack.hgrid;

/// HHisItem is used to model a timestamp/value pair
///
/// See [Project Haystack](http://project-haystack.org/doc/Ops#hisRead)
class HHisItem extends HDict {
  /// Map [HGrid] to [HHisItem].  Grid must have ts and val columns.
  static List<HHisItem> gridToItems(HGrid grid) {
    HCol ts = grid.colByName("ts");
    HCol val = grid.colByName("val");
    List<HHisItem> items = new List<HHisItem>(grid.numRows);
    for (int i = 0; i < items.length; ++i) {
      HRow row = grid.row(i);
      items[i] = new HHisItem(row.getByCol(ts, checked: true) as HDateTime,
          row.getByCol(val, checked: false));
    }
    return items;
  }

  /// Construct from timestamp, value
  factory HHisItem(HDateTime ts, HVal val) {
    if (ts == null || val == null) throw new ArgumentError("ts or val is null");
    return new HHisItem._(ts, val);
  }

  HHisItem._(HDateTime this.ts, HVal this.val) : super._();

  /// Timestamp of history sample
  final HDateTime ts;

  /// Value of history sample
  final HVal val;

  @override
  int get size => 2;

  @override
  HVal get(String name, {bool checked: true}) {
    if (name == "ts") return ts;
    if (name == "val") return val;
    if (!checked) return null;
    throw new UnknownNameError(name);
  }

  @override
  Iterator<DictEntry> iterator() => new _HisItemIterator(ts, val);

  @override
  HVal operator [](Object key) => get(key.toString(), checked: false);

  @override
  Iterable<String> get keys => ["ts", "val"];
}

class _HisItemIterator implements Iterator<DictEntry> {
  _HisItemIterator(this._ts, this._val);

  /// Timestamp of history sample
  final HDateTime _ts;

  /// Value of history sample
  final HVal _val;

  int _currentId = -1;
  DictEntry _current;

  @override
  bool moveNext() {
    if (_currentId >= 1) return false;

    _currentId++;

    if (_currentId == 0) {
      _current = new DictEntry('ts', _ts);
    } else if (_currentId == 1) {
      _current = new DictEntry('val', _val);
    }

    return true;
  }

  @override
  DictEntry get current => _current;
}
