///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hxstr;

import 'package:haystack/src/string_utils.dart';
import 'package:unicode/unicode.dart' as unicode;

import 'HBin.dart';
import 'HVal.dart';

/// XStr is an extended string which is a type name and value
/// encoded as a string.
///
/// It is used as a generic value when an XStr is decoded without
/// any predefined type.
class HXStr extends HVal {
  static HVal decode(String type, String val) {
    if ("Bin" == type) return new HBin(val);
    return new HXStr._(type, val);
  }

  static HXStr encode(Object val) {
    return new HXStr._(val.runtimeType.toString(), val.toString());
  }

  /// Type name
  final String type;

  /// String value
  final String val;

  @override
  int get hashCode {
    int result = type.hashCode;
    result = 31 * result + val.hashCode;
    return result;
  }


  HXStr._(String this.type, String this.val) {
    if (!_isValidType(type))
      throw new ArgumentError("Invalid type name: $type");
  }


  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    s..write(type)..write("(\"")..write(val)..write("\")");
    return s.toString();
  }

  @override
  String toJson() {
    throw new UnsupportedError("toJson");
  }

  @override
  bool operator ==(Object o) {
    if (o == null || runtimeType != o.runtimeType) return false;

    HXStr hxStr = o as HXStr;

    if (type != hxStr.type) return false;
    return val == hxStr.val;
  }


  static bool _isValidType(String t) {
    if (t == null || t.isEmpty || !unicode.isUppercaseLetter(t.codeUnitAt(0)))
      return false;
    for (var c in t.runes) {
      if (isLetter(c)) continue;
      if (unicode.isDecimalNumber(c)) continue;
      if (c == '_') continue;
      return false;
    }
    return true;
  }
}
