///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.token;

class HaystackToken {
  static final HaystackToken eof = new HaystackToken("eof");

  static final HaystackToken id = new HaystackToken("identifier");
  static final HaystackToken num = new HaystackToken("Number", true);
  static final HaystackToken str = new HaystackToken("Str", true);
  static final HaystackToken ref = new HaystackToken("Ref", true);
  static final HaystackToken uri = new HaystackToken("Uri", true);
  static final HaystackToken date = new HaystackToken("Date", true);
  static final HaystackToken time = new HaystackToken("Time", true);
  static final HaystackToken dateTime = new HaystackToken("DateTime", true);

  static final HaystackToken colon = new HaystackToken(":");
  static final HaystackToken comma = new HaystackToken(",");
  static final HaystackToken semicolon = new HaystackToken(";");
  static final HaystackToken minus = new HaystackToken("-");
  static final HaystackToken eq = new HaystackToken("==");
  static final HaystackToken notEq = new HaystackToken("!=");
  static final HaystackToken lt = new HaystackToken("<");
  static final HaystackToken lt2 = new HaystackToken("<<");
  static final HaystackToken ltEq = new HaystackToken("<=");
  static final HaystackToken gt = new HaystackToken(">");
  static final HaystackToken gt2 = new HaystackToken(">>");
  static final HaystackToken gtEq = new HaystackToken(">=");
  static final HaystackToken lbracket = new HaystackToken("[");
  static final HaystackToken rbracket = new HaystackToken("]");
  static final HaystackToken lbrace = new HaystackToken("{");
  static final HaystackToken rbrace = new HaystackToken("}");
  static final HaystackToken lparen = new HaystackToken("(");
  static final HaystackToken rparen = new HaystackToken(")");
  static final HaystackToken arrow = new HaystackToken("->");
  static final HaystackToken slash = new HaystackToken("/");
  static final HaystackToken assign = new HaystackToken("=");
  static final HaystackToken bang = new HaystackToken("!");
  static final HaystackToken nl = new HaystackToken("newline");

  HaystackToken(String this.symbol, [bool literal])
      : this.literal = literal ?? false;

  @override
  bool operator ==(Object that) {
    return that is HaystackToken &&
        literal == that.literal &&
        symbol == that.symbol;
  }

  @override
  int get hashCode {
    int result = symbol.hashCode;
    result = 31 * result + (literal ? 1 : 0);
    return result;
  }

  @override
  String toString() => symbol;

  final String symbol;
  final bool literal;
}
