library haystack.src.utils.simplify;

import 'package:haystack/core.dart';


/// Makes HVals more simpler, for commit call.
///
/// Makes ids ([HRef]) relative and removes dis part.
T simplifyVal<T extends HVal>(T val) {
  if (val is HRef) {
    return new HRef(val.val.substring(val.val.lastIndexOf(':')+1)) as T;
  }

  if (val is HGrid) {
    var rows = val.map((row) => simplifyVal<HRow>(row)).toList();
    return HGridBuilder.dictsToGrid(val.meta, rows) as T;
  }

  if (val is HRow) {
    var newValues = val.values.map((v) => simplifyVal<HVal>(v)).toList();
    return new HRow(val.grid, newValues) as T;
  }

  if (val is HDict && val is! HHisItem) {
    var newMap = <String, HVal>{};
    val.keys.forEach((key) {
      newMap[key] = simplifyVal<HVal>(val[key]);
    });

    return new HDict(newMap) as T;
  }

  return val;
}

Iterable<T> simplifyList<T extends HVal>(Iterable<T> values) {
  return values.map<T>((val) => simplifyVal<T>(val));
}