///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.row_test;

import 'package:haystack/core.dart';
import 'package:test/test.dart';

void main () {
  group('Value test', () {
    test('hval', () {
      HGridBuilder aBuilder = new HGridBuilder();
      aBuilder.addCol("id");
      aBuilder.addCol("dis");
      aBuilder.addCol("area");
      aBuilder.addRow([new HRef("a"), new HStr("Alpha"), new HNum(1200)]);
      aBuilder.addRow([new HRef("b"), new HStr("Beta"), null ]);

      HGridBuilder bBuilder = new HGridBuilder();
      bBuilder.addCol("id");
      bBuilder.addCol("dis");
      bBuilder.addCol("area");
      bBuilder.addRow([new HRef("a"), new HStr("Alpha"), new HNum(1200)]);
      bBuilder.addRow([new HRef("b"), new HStr("Beta"), aBuilder.toGrid() ]);

      HGrid bGrid = bBuilder.toGrid();
      HRow r = bGrid.row(1);

      expect(r, isNotNull);
      expect(r, equals(r));
      expect(r == r, isTrue);
    });
  });

  group('Test simple', () {
    HGridBuilder b = new HGridBuilder();
    b.addCol("id");
    b.addCol("dis");
    b.addCol("area");
    b.addRow([new HRef("a"), new HStr("Alpha"), new HNum(1200)]);
    b.addRow([new HRef("b"), new HStr("Beta"), null ]);

    HGrid g = b.toGrid();
    HRow r = g.row(0);

    test('row is map', () {
        expect(r is Map, equals(true));
    });

    test('row as map', () {
      expect(r.isEmpty, equals(false));
      expect(r.isNotEmpty, equals(true));
      expect(r.containsKey("id"), equals(true));
      expect(r.containsKey("dis"), equals(true));
      expect(r["dis"].toString(), equals("Alpha"));
    });

    test('row forEach', () {
      r.forEach(( String k, HVal v ) {
        expect( r[k], equals( v ));
      });
    });
  });
}
